from ....functions.builtins import instance_pyobj_by_mod_path
from ....functions.decorators import instance_he_by_module_name, \
    parse_self_x_item_value_or_set_by_instance_he_by_module_name, set_self_x_item_value_by_setter_with_agr, \
    parse_self_x_item_value_or_set_by_func
from ...types.he import He


class Htmdoc(He):
    def __init__(self, u, configs:dict=None):
        He.__init__(self, u)
        self.configs.update(configs)

    @property
    @parse_self_x_item_value_or_set_by_func()
    def elements(self): return {}

    @property
    @parse_self_x_item_value_or_set_by_instance_he_by_module_name()
    def configs(self): return

    @property
    @parse_self_x_item_value_or_set_by_instance_he_by_module_name(module_name='tags.html')
    def root(self): return

    @instance_he_by_module_name(module_name='nodes#')
    def comment(self, content: str = None): return
    @instance_he_by_module_name(module_name='nodes#')
    def script(self, content: str = None): return
    @instance_he_by_module_name(module_name='nodes#')
    def text(self, content: str = None): return
    @instance_he_by_module_name(module_name='nodes#')
    def wrap(self, content=None): return

    @staticmethod
    def ___auto_create_tag___(u, name:str, args:tuple|list, kwargs:dict):
        # return self.__b__.pyobj_by_modnam(f'{Htmdoc.__module__.rsplit('.', 1)[0]}.tags.{name.lower()}', [u, *args], kwargs)
        try: return instance_pyobj_by_mod_path(f'{Htmdoc.__module__.rsplit('.', 1)[0]}.tags.{name.lower()}', [u, *args], kwargs)
        except: return instance_pyobj_by_mod_path(f'{Htmdoc.__module__.rsplit('.', 1)[0]}.nodes.tag', [u, name.lower(), *args], kwargs)


    def __getattribute__(self, name:str):
        def handler(*args, **kwargs):
            return self.___auto_create_tag___(self, name, args, kwargs)
        if name and name[0].isupper():
            return handler
        return super().__getattribute__(name)

    """def __getattr__(self, name:str):
        # 当访问不存在的属性或方法时触发
        def handler(*args, **kwargs): return self.___auto_create_tag___(self, name, args, kwargs)
        return handler"""

    @property
    @parse_self_x_item_value_or_set_by_func()
    def ___output_next_break___(self): return False
    @___output_next_break___.setter
    @set_self_x_item_value_by_setter_with_agr(value_type='bol')
    def ___output_next_break___(self, value): pass

    def output(self):
        """s = '['
        elements = self.elements
        for k, v in elements.items(): s += f'\n\t{k}: {v}'
        s += ']'
        s += '\n' + str(self)
        return s"""
        return self.root.output()
    def __str__(self):
        return self.root.__str__()


    @property
    @parse_self_x_item_value_or_set_by_func()
    def ___css_links___(self): return []
    @property
    @parse_self_x_item_value_or_set_by_func()
    def ___js_links___(self): return []
    @property
    @parse_self_x_item_value_or_set_by_func()
    def ___css_scripts___(self): return []
    @property
    @parse_self_x_item_value_or_set_by_func()
    def ___js_scripts___(self): return []

    def add_css_link(self, url):
        if url not in self.___css_links___: self.___css_links___.append(url)
    def add_js_link(self, url):
        if url not in self.___js_links___:
            self.___js_links___.append(url)
    def add_css_code(self, code): self.___css_scripts___.append(code)
    def add_js_code(self, code): self.___js_scripts___.append(code)

    """def add_library_css_link(self, *names): self.add_css_link(self.library_file_url(*names))
    def add_library_js_link(self, *names):self.add_js_link(self.library_file_url(*names))

    def library_file_url(self, *names): return self.__b__.join_url(self.configs.library_url_root, *names)
    def library_file(self, *names): return self.__s__.modules.file(str(os.path.join(os.path.dirname(__file__), 'library', *names)))"""

    def add_css_link_node(self, url: str):
        n = self.root.head.inner_after(-100).Link(url)
        n.attributes['rel'] = 'stylesheet'
        n.attributes['type'] = 'text/css'
        n.attributes['crossorigin'] = 'anonymous'
        return n
    def add_js_link_node(self, url: str):
        n = self.root.body.after(-1000).Script()
        n.attributes['src'] = url
        n.attributes['type'] = 'text/javascript'
        return n
    def add_css_script_node(self, script: str):
        return self.root.head.after(-100).Style(script)
    def add_js_script_node(self, script: str):
        n = self.root.body.after(-1000).Script()
        n.script(script)
        n.attributes['type'] = 'text/javascript'
        return n



from ..cls import Cls
from ..core import CssMaker
from ....types.he import He


class Med(He):
    @staticmethod
    def ___make_name___(media_min_width:int=None, media_max_width:int=None, media_min_height:int=None, media_max_height:int=None)->str: ...

    def __init__(self, u, name: str, media_min_width:int=None, media_max_width:int=None, media_min_height:int=None, media_max_height:int=None): ...

    @property
    def __u__(self)-> CssMaker: ...

    @property
    def name(self)-> str: ...

    @property
    def clses(self)-> dict[str, Cls]: ...
    def cls(self, name:str)-> Cls: ...



from ... import Htmdoc
from ..input import Input
from ...nodes.wrap import Wrap


class InputText(Input):
    def __init__(self, u: Htmdoc | Wrap, name: str = None, value: str = None, placeholder:str=None, attributes: dict=None): ...



class Orders(dict):
    def __init__(self, orders=None): ...

    def add(self, column: str, by: bool | str = None) -> None: ...

    def adds(self, data: dict = None) -> None: ...

    def output(self) -> str: ...



from .. import System
from ..components.configs import Configs as ModConfigs


class Configs(ModConfigs):
    def __init__(self, s: System): ...

    @property
    def language(self)-> str: ...
    @language.setter
    def language(self, value) -> None: ...

    @property
    def debug_prefix(self)-> str: ...
    @debug_prefix.setter
    def debug_prefix(self, value) -> None: ...
    @property
    def debug_printable(self)-> bool: ...
    @debug_printable.setter
    def debug_printable(self, value) -> None: ...
    @property
    def debug_saveable(self)-> bool: ...
    @debug_saveable.setter
    def debug_saveable(self, value) -> None: ...



from .....functions.builtins import parse_mod_path_by_pyobj
from .....functions.decorators import parse_self_x_item_value_or_set_by_func, parse_self_x_item_value


class I:
    def __init__(self, he):
        self.__x = {}
        self.__x__['__he__'] = he
    @property
    def __x__(self): return self.__x
    @property
    @parse_self_x_item_value()
    def __he__(self): return

    @property
    def id(self): return self.__he__.__id__
    @property
    @parse_self_x_item_value_or_set_by_func()
    def mod_path(self): return self.__he__.__s__.components.mod_path(parse_mod_path_by_pyobj(self.__he__))
    @property
    def mod_type(self): return self.mod_path.mod_type
    @property
    def cls_type(self): return self.mod_path.cls_type
    @property
    @parse_self_x_item_value_or_set_by_func()
    def obj_type(self): return self.__he__.__s__.components.obj_type(self.__he__)

    @property
    def folder(self): return self.cls_type.folder



from ..components.debugs import Debugs as ModDebugs


class Debugs(ModDebugs):
    def __init__(self, s):
      super().__init__(s)
      self.configs.hook('prefix', s.configs, 'debug_prefix')
      self.configs.hook('printable', s.configs, 'debug_printable')
      self.configs.hook('saveable', s.configs, 'debug_saveable')



from ..med import Med
from .....functions.decorators import parse_self_x_item_value, parse_self_x_item_value_or_set_by_func
from ....types.he import He


class Cls(He):
    def __init__(self, u, name: str):
        He.__init__(self, u)
        self.__x__['name'] = name

    @property
    @parse_self_x_item_value()
    def name(self): return

    @property
    @parse_self_x_item_value_or_set_by_func()
    def atrs(self): return {}

    def atr(self, name=None, value=None): self.atrs[name] = value
    def atr_px(self, name, value): self.atr(name, f'{value}px')

    def __str__(self):
        ls = []
        prefix = '\t' if isinstance(self.__u__, Med) else ''
        ls.append(f'{prefix}{self.name}{{')
        for n, v in self.atrs.items():
            ls.append(f'{prefix}\t{n}: {v};')
        ls.append(f'{prefix}}}')
        return '\n'.join(ls)



from .....functions.decorators import set_self_item_value_by_setter_with_agr, parse_self_item_value
from ...configs import Configs as ModConfigs
from ....types.he import He


class Configs(ModConfigs):
    def __init__(self, u: He):
        super().__init__(u, {
            "connection": "mysql",

            "host": "",
            "port": 3306,
            "user": "",
            "password": "",
            "dbname": "",
            "prefix": "",

            "max_connections": 30,
            "connect_timeout": 5,
            "column_key_sn": "_sn",
            "column_key_id": "_id",
            "column_key_az": "_az",
            "column_key_kw": "_kw",
            "column_key_ur": "_ur",
            "column_key_dc": "_dc",
            "column_key_du": "_du",
            "data_admin_id": "_admin",
        })

    @property
    @parse_self_item_value()
    def connection(self): return
    @connection.setter
    @set_self_item_value_by_setter_with_agr(value_type='strue')
    def connection(self, value): pass
    @property
    @parse_self_item_value()
    def host(self): return
    @host.setter
    @set_self_item_value_by_setter_with_agr(value_type='strue')
    def host(self, value): pass
    @property
    @parse_self_item_value()
    def port(self): return
    @port.setter
    @set_self_item_value_by_setter_with_agr(value_type='intp')
    def port(self, value): pass
    @property
    @parse_self_item_value()
    def user(self): return
    @user.setter
    @set_self_item_value_by_setter_with_agr(value_type='strue')
    def user(self, value): pass
    @property
    @parse_self_item_value()
    def password(self): return
    @password.setter
    @set_self_item_value_by_setter_with_agr(value_type='strue')
    def password(self, value): pass
    @property
    @parse_self_item_value()
    def dbname(self): return
    @dbname.setter
    @set_self_item_value_by_setter_with_agr(value_type='strue')
    def dbname(self, value): pass
    @property
    @parse_self_item_value()
    def prefix(self): return
    @prefix.setter
    @set_self_item_value_by_setter_with_agr(value_type='strue')
    def prefix(self, value): pass
    @property
    @parse_self_item_value()
    def max_connections(self): return
    @max_connections.setter
    @set_self_item_value_by_setter_with_agr(value_type='intp')
    def max_connections(self, value): pass
    @property
    @parse_self_item_value()
    def connect_timeout(self): return
    @connect_timeout.setter
    @set_self_item_value_by_setter_with_agr(value_type='intp')
    def connect_timeout(self, value): pass
    @property
    @parse_self_item_value()
    def column_key_sn(self): return
    @column_key_sn.setter
    @set_self_item_value_by_setter_with_agr(value_type='strue')
    def column_key_sn(self, value): pass
    @property
    @parse_self_item_value()
    def column_key_id(self): return
    @column_key_id.setter
    @set_self_item_value_by_setter_with_agr(value_type='strue')
    def column_key_id(self, value): pass
    @property
    @parse_self_item_value()
    def column_key_az(self): return
    @column_key_az.setter
    @set_self_item_value_by_setter_with_agr(value_type='strue')
    def column_key_az(self, value): pass
    @property
    @parse_self_item_value()
    def column_key_kw(self): return
    @column_key_kw.setter
    @set_self_item_value_by_setter_with_agr(value_type='strue')
    def column_key_kw(self, value): pass
    @property
    @parse_self_item_value()
    def column_key_ur(self): return
    @column_key_ur.setter
    @set_self_item_value_by_setter_with_agr(value_type='strue')
    def column_key_ur(self, value): pass
    @property
    @parse_self_item_value()
    def column_key_dc(self): return
    @column_key_dc.setter
    @set_self_item_value_by_setter_with_agr(value_type='strue')
    def column_key_dc(self, value): pass
    @property
    @parse_self_item_value()
    def column_key_du(self): return
    @column_key_du.setter
    @set_self_item_value_by_setter_with_agr(value_type='strue')
    def column_key_du(self, value): pass
    @property
    @parse_self_item_value()
    def data_admin_id(self): return
    @data_admin_id.setter
    @set_self_item_value_by_setter_with_agr(value_type='strue')
    def data_admin_id(self, value): pass



from types import ModuleType
from ...types.he import He
from ...types.hs import Hs
from ....functions.builtins import (
    parse_mod_path_by_pycls, parse_mod_path_by_pyobj, import_pymod_by_mod_path, import_pycls_by_mod_path,
    parse_mod_path_by_rel_path
)
from ....functions.decorators import parse_self_x_item_value_or_set_by_func


class ModPath(Hs):
    def __new__(cls, u, name: str | ModuleType | type | He):
        if isinstance(name, He):
            obj = Hs.__new__(cls, u, parse_mod_path_by_pyobj(name))
        elif isinstance(name, type) and issubclass(name, He):
            obj = Hs.__new__(cls, u, parse_mod_path_by_pycls(name))
        elif isinstance(name, ModuleType):
            obj = Hs.__new__(cls, u, name.__name__)
        elif isinstance(name, str):
            obj = Hs.__new__(cls, u, name)
        else:
            raise ValueError(f'ModName 的参数 name 只允许： He | types.ModuleType | type(He) | str')
        return obj

    def __init__(self, u, name: str | ModuleType | type | He):
        if isinstance(name, He):
            Hs.__init__(self, u, parse_mod_path_by_pyobj(name))
            self.__x__['pycls'] = name.__class__
            self.__x__['pymod'] = name.__module__
        elif isinstance(name, type) and issubclass(name, He):
            Hs.__init__(self, u, parse_mod_path_by_pycls(name))
            self.__x__['pycls'] = name
            self.__x__['pymod'] = name.__module__
        elif isinstance(name, ModuleType):
            Hs.__init__(self, u, name.__name__)
            self.__x__['pymod'] = name
        elif isinstance(name, str):
            Hs.__init__(self, u, name)

    @property
    @parse_self_x_item_value_or_set_by_func()
    def pymod(self):
        try: return import_pymod_by_mod_path(self)
        except: return None
    @property
    @parse_self_x_item_value_or_set_by_func()
    def pycls(self):
        try: return import_pycls_by_mod_path(self)
        except: return None

    @property
    @parse_self_x_item_value_or_set_by_func()
    def mod_type(self):
        if self.pycls: return self.__sm__.mod_type(self.pycls)
        if self.pymod: return self.__sm__.mod_type(self.pymod)
        return None

    @property
    @parse_self_x_item_value_or_set_by_func()
    def cls_type(self):
        if self.pycls: return self.__sm__.cls_type(self.pycls)
        return None

    def rel_mod_path(self, rel_path=None):
        return ModPath(self.__s__, parse_mod_path_by_rel_path(self, rel_path))



from ....functions.decorators import parse_self_x_item_value_or_set_by_func, \
    parse_self_x_item_value_or_set_by_instance_he_by_module_name
from ...types.he import He


class Database(He):
    def __init__(self, u, host=None, user=None, password=None, dbname=None, port=None, prefix=None, connection=None, configs:dict=None):
        He.__init__(self, u)
        self.configs.update(configs)
        self.configs['host'] = host
        self.configs['user'] = user
        self.configs['password'] = password
        self.configs['dbname'] = dbname
        self.configs['port'] = port or 3306
        self.configs['prefix'] = prefix or ''
        self.configs['connection'] = connection or 'mysql'

    @property
    @parse_self_x_item_value_or_set_by_instance_he_by_module_name()
    def configs(self): return

    @property
    @parse_self_x_item_value_or_set_by_instance_he_by_module_name()
    def tables(self): return

    @property
    @parse_self_x_item_value_or_set_by_func()
    def connection(self):
        if self.configs['connection'] == 'mysql':
            from .connections.mysql import Mysql
            return Mysql(self)
        return None



from .address import Address
from ... import WebSystem
from .....types.hd import Hd
from ...configs import Configs
from ...clients import Clients
from .request import Request


class Client(Hd):
    @property
    def __websystem__(self)-> WebSystem: ...
    @property
    def __clients__(self)-> Clients: ...
    @property
    def __configs__(self)-> Configs: ...

    @property
    def uid(self)-> str: ...
    @property
    def expiry(self)-> int: ...
    @property
    def request(self)-> Request: ...
    def hook_session(self, session)-> None: ...
    @property
    def session(self)-> dict: ...

    def address(self, mode: str = None, module: str = None, arguments: dict[str,str] = None, keyword: str = None, page: int = None, orders: dict[str,str] = None, filters: dict[str,str] = None, items_per_page: int = None, pages_per_group: int = None, reference: str = None, language: str = None, security: str = None)-> Address: ...
    def url(self, mode: str = None, module: str = None, arguments: dict[str,str] = None, keyword: str = None, page: int = None, orders: dict[str,str] = None, filters: dict[str,str] = None, items_per_page: int = None, pages_per_group: int = None, reference: str = None, language: str = None, security: str = None)-> str: ...
    def redirect(self, mode: str = None, module: str = None, arguments: dict[str,str] = None, keyword: str = None, page: int = None, orders: dict[str,str] = None, filters: dict[str,str] = None, items_per_page: int = None, pages_per_group: int = None, reference: str = None, language: str = None, security: str = None): ...



from .database.core import Database
from ..folder import Folder
from ..mod_path import ModPath
from ..mod_type import ModType
from ...types.he import He
from .configs import Configs
from .server import Server
from .clients import Clients


class WebSystem(He):
    def __init__(self, senior: He, module: str, host: str = None, port: int = None, db_host: str=None, db_user: str=None, db_password: str=None, db_dbname: str=None, db_port: int=None, db_prefix: str=None, configs:dict=None): ...
    @property
    def __debugs_prefix__(self)-> list[str]: ...
    @property
    def configs(self)-> Configs: ...
    @property
    def database(self)-> Database: ...
    @property
    def server(self)-> Server: ...
    @property
    def clients(self)-> Clients: ...
    @property
    def website_mod_path(self)-> ModPath | None: ...
    @property
    def website_mod_type(self)-> ModType | None: ...
    def is_website_exists(self)-> bool: ...
    @property
    def website_folder(self)-> Folder | None: ...



from ...node import Node


class Wrap(Node):
    def __init__(self, u, content=None):
        super().__init__(u)
        self.append(content)

    @property
    def juniors(self): return self.___juniors___

    @staticmethod
    def is_wrap(): return True

    def comment(self, content: str = None): return self.___comment___(content)
    def script(self, content: str = None): return self.___script___(content)
    def text(self, content: str = None): return self.___text___(content)
    def wrap(self, content=None): return self.___wrap___(content)

    def append(self, element=None): return self.___append___(element)

    def __getattr__(self, name:str):
        if name and name[0].isupper():
            def handler(*args, **kwargs):
                return self.document.___auto_create_tag___(self, name, args, kwargs)
            return handler
        raise AttributeError(f"'{type(self).__name__}' object has no attribute '{name}'")

    @property
    def anchored(self): return self.___anchored_to___
    def is_anchored(self): return self.___is_anchored_to___()



from ...types.he import He
from ...types.hs import Hs
from ..file import File
from ..folder import Folder


class FileFolder(Hs):
    def __init__(self, u: He, path: str): ...
    @property
    def name(self)-> str: ...

    @property
    def parent(self) -> Folder: ...
    def brother_file(self, name: str) -> File: ...
    def brother_folder(self, name: str) -> Folder: ...



from ... import Htmdoc
from ..wrap import Wrap


class Tag(Wrap):
    def __init__(self, u: Htmdoc | Wrap, name: str=None, content=None, attributes:dict[str, str|int|float|None]=None) -> None: ...

    @property
    def name(self)-> str: ...
    @property
    def attributes(self)-> dict[str, str|int|float|None]: ...

    def inner_before(self, sn:int)-> Wrap: ...
    @property
    def inner_before1(self)-> Wrap: ...
    @property
    def inner_before2(self)-> Wrap: ...
    @property
    def inner_before3(self)-> Wrap: ...
    @property
    def inner_before4(self)-> Wrap: ...
    @property
    def inner_before5(self)-> Wrap: ...
    def inner_after(self, sn:int)-> Wrap: ...
    @property
    def inner_after1(self)-> Wrap: ...
    @property
    def inner_after2(self)-> Wrap: ...
    @property
    def inner_after3(self)-> Wrap: ...
    @property
    def inner_after4(self)-> Wrap: ...
    @property
    def inner_after5(self)-> Wrap: ...

    @property
    def output_pairable(self)-> bool: ...
    @property
    def output_brealable(self)-> bool: ...



from .......types.he import He
from ..... import Database
from .... import Tables
from ... import Table
from .. import Columns
from .....connection import Connection
from .....configs import Configs


class Column(He):
    def __init__(self, u: Columns, name: str): ...
    @property
    def __database__(self)-> Database: ...
    @property
    def __tables__(self)-> Tables: ...
    @property
    def __table__(self)-> Table: ...
    @property
    def __columns__(self)-> Columns: ...
    @property
    def __connection__(self)-> Connection: ...
    @property
    def __configs__(self)-> Configs: ...
    @property
    def name(self)-> str: ...

    def exists(self, name: str)-> bool: ...
    def create(self, datatype: str = None, length: int = 0, unique: bool = False, nullable: bool = False) -> bool: ...
    def delete(self, name: str)-> bool: ...

    def set_unique(self, name: str)-> bool: ...



from .....functions.builtins import join_url
from .....functions.decorators import parse_self_x_item_value_or_set_by_func
from ....types.he import He


class Server(He):
    def __init__(self, system):
        super().__init__(system)

    @property
    def __websystem__(self): return self.__u__
    @property
    def __configs__(self): return self.__websystem__.configs
    def __deubg_warning__(self, content): raise RuntimeError('函数必须在继承类中重写')
    def __deubg_info__(self, content): raise RuntimeError('函数必须在继承类中重写')
    def make_response_text_html(self, code: str): raise RuntimeError('函数必须在继承类中重写')
    def make_response_text_plain(self, text: str): raise RuntimeError('函数必须在继承类中重写')
    def make_response_text_css(self, code: str): raise RuntimeError('函数必须在继承类中重写')
    def make_response_text_csv(self, code: str): raise RuntimeError('函数必须在继承类中重写')
    def make_response_text_markdown(self, code: str): raise RuntimeError('函数必须在继承类中重写')
    def make_response_text_xml(self, code: str): raise RuntimeError('函数必须在继承类中重写')
    # noinspection PyMethodMayBeStatic
    def make_response_error(self, title=None, heading=None, content=None):
        title = title or 'Error'
        heading = heading or 'Unknown Error Occurred'
        content = content or 'The requested URL was not found on the server. If you entered the URL manually please check your spelling and try again.'
        return f'<!doctype html><html lang=en><title>{title}</title><h1>{heading}</h1><p>{content}</p>'

    def make_response_redirect(self, url): raise RuntimeError('函数必须在继承类中重写')
    def make_response_file(self, file, as_attachment=False, mimetype=None, download_name=None): raise RuntimeError('函数必须在继承类中重写')
    def make_response_404(self): raise RuntimeError('函数必须在继承类中重写')
    @property
    @parse_self_x_item_value_or_set_by_func()
    def root_url(self): return f'{self.__configs__.url_root_path.strip('/')}'
    def generate_url(self, *names): return join_url(self.root_url, *names)

    def run(self): raise RuntimeError('函数必须在继承类中重写')



from .number import Number
from ..he import He


class Hf(He, float, Number):
    def __new__(cls, u: He, number: str | float | int): ...
    def __init__(self, u: He, number: str | float | int): ...



from .....functions.decorators import parse_self_item_value, set_self_item_value_by_setter_with_agr
from ...configs import Configs as PluCfg


class Configs(PluCfg):
    @property
    @parse_self_item_value()
    def website_module(self): return
    @website_module.setter
    @set_self_item_value_by_setter_with_agr(value_type='strue')
    def website_module(self, value): pass

    @property
    @parse_self_item_value()
    def mainpy(self): return
    @mainpy.setter
    @set_self_item_value_by_setter_with_agr(value_type='strue')
    def mainpy(self, value): pass

    @property
    @parse_self_item_value()
    def exe_name(self): return
    @exe_name.setter
    @set_self_item_value_by_setter_with_agr(value_type='strue')
    def exe_name(self, value): pass

    @property
    @parse_self_item_value()
    def show_console(self): return
    @show_console.setter
    @set_self_item_value_by_setter_with_agr(value_type='bol')
    def show_console(self, value): pass

    @property
    @parse_self_item_value()
    def one_file(self): return
    @one_file.setter
    @set_self_item_value_by_setter_with_agr(value_type='bol')
    def one_file(self, value): pass

    @property
    @parse_self_item_value()
    def one_dir(self): return
    @one_dir.setter
    @set_self_item_value_by_setter_with_agr(value_type='bol')
    def one_dir(self, value): pass

    @property
    @parse_self_item_value()
    def pack_overwrite(self): return
    @pack_overwrite.setter
    @set_self_item_value_by_setter_with_agr(value_type='bol')
    def pack_overwrite(self, value): pass

    @property
    @parse_self_item_value()
    def root_folders(self): return
    @root_folders.setter
    @set_self_item_value_by_setter_with_agr(value_type='lst')
    def root_folders(self, value): pass

    @property
    @parse_self_item_value()
    def root_modules(self): return
    @root_modules.setter
    @set_self_item_value_by_setter_with_agr(value_type='lst')
    def root_modules(self, value): pass

    @property
    @parse_self_item_value()
    def dist_path(self): return
    @dist_path.setter
    @set_self_item_value_by_setter_with_agr()
    def dist_path(self, value): pass

    @property
    @parse_self_item_value()
    def build_path(self): return
    @build_path.setter
    @set_self_item_value_by_setter_with_agr()
    def build_path(self, value): pass

    @property
    @parse_self_item_value()
    def spec_path(self): return
    @spec_path.setter
    @set_self_item_value_by_setter_with_agr()
    def spec_path(self, value): pass

    @property
    @parse_self_item_value()
    def comd_path(self): return
    @comd_path.setter
    @set_self_item_value_by_setter_with_agr()
    def comd_path(self, value): pass

    @property
    @parse_self_item_value()
    def icon_file(self): return
    @icon_file.setter
    @set_self_item_value_by_setter_with_agr()
    def icon_file(self, value): pass

    @property
    @parse_self_item_value()
    def version_file(self): return
    @version_file.setter
    @set_self_item_value_by_setter_with_agr()
    def version_file(self, value): pass



from .......functions.decorators import parse_self_x_item_value, set_self_item_value_by_setter_with_agr
from .....configs import Configs


class Settings(Configs):
    def __init__(self, u):
        super().__init__(u, file=u.data_settings_file, defaults={
            'page_feature': 'general',
            'power': '',
            'icon': 'fa-code',
            'page_layout': '',
        })

    @property
    @parse_self_x_item_value()
    def page_feature(self): return
    @page_feature.setter
    @set_self_item_value_by_setter_with_agr(value_type='str')
    def page_feature(self, value): pass
    def is_page_feature_general(self): return not self.page_feature or self.page_feature == self.__s__.definitions.webapp_page_features.general
    def is_page_feature_sign_in(self): return self.page_feature == self.__s__.definitions.webapp_page_features.sign_in
    def is_page_feature_sign_out(self): return self.page_feature == self.__s__.definitions.webapp_page_features.sign_out
    def is_page_feature_sign_reset(self): return self.page_feature == self.__s__.definitions.webapp_page_features.sign_reset
    def is_page_feature_sign_up(self): return self.page_feature == self.__s__.definitions.webapp_page_features.sign_up

    @property
    @parse_self_x_item_value()
    def power(self): return
    @power.setter
    @set_self_item_value_by_setter_with_agr(value_type='str')
    def power(self, value): pass
    def is_power_public(self): return self.power == self.__s__.definitions.webapp_power_types.public
    def is_power_groups(self): return self.power == self.__s__.definitions.webapp_power_types.groups
    def is_power_inherited(self): return self.power == self.__s__.definitions.webapp_power_types.inherited
    def is_power_members(self): return self.power == self.__s__.definitions.webapp_power_types.members

    @property
    @parse_self_x_item_value()
    def icon(self): return
    @icon.setter
    @set_self_item_value_by_setter_with_agr(value_type='str')
    def icon(self, value): pass

    @property
    @parse_self_x_item_value()
    def page_layout(self): return
    @page_layout.setter
    @set_self_item_value_by_setter_with_agr(value_type='str')
    def page_layout(self, value): pass



from ..clients.client.address import Address
from ....types.hd import Hd
from .. import WebSystem
from ..clients.client import Client
from .webmod import Webmod
from .htmdoc import Htmdoc


class Website(Hd):
    def __init__(self, websystem: WebSystem, client: Client): ...

    @property
    def websystem(self) -> WebSystem: ...
    @property
    def client(self) -> Client: ...
    @property
    def htmdoc(self) -> Htmdoc: ...
    @property
    def favicon_ico_url_path(self)-> str: ...
    def webmod(self, *names: str)-> Webmod: ...
    @property
    def webroot(self)-> Webmod: ...

    def address(self, mode: str = None, module: str = None, arguments: dict[str,str] = None, keyword: str = None, page: int = None, orders: dict[str,str] = None, filters: dict[str,str] = None, items_per_page: int = None, pages_per_group: int = None, reference: str = None, language: str = None, security: str = None)-> Address: ...
    def url(self, mode: str = None, module: str = None, arguments: dict[str,str] = None, keyword: str = None, page: int = None, orders: dict[str,str] = None, filters: dict[str,str] = None, items_per_page: int = None, pages_per_group: int = None, reference: str = None, language: str = None, security: str = None)-> str: ...
    def redirect(self, mode: str = None, module: str = None, arguments: dict[str,str] = None, keyword: str = None, page: int = None, orders: dict[str,str] = None, filters: dict[str,str] = None, items_per_page: int = None, pages_per_group: int = None, reference: str = None, language: str = None, security: str = None): ...

    def response(self): ...




from ..input import Input


class InputText(Input):
    def __init__(self, u, name: str = None, value: str = None, placeholder:str=None, attributes:dict[str, str|int|float|None]=None):
        if placeholder:
            if isinstance(attributes, dict):
                attributes['placeholder'] = placeholder
            else:
                attributes = {'placeholder': placeholder}
        Input.__init__(self, u, name, value, attributes)




from io import BytesIO
from PIL import Image as PilImage
from ...types.he import He


ICO_CLOSE = ...
ICO_DEMO = ...
ICO_DOCUMENT = ...
ICO_HELP = ...
ICO_HOME = ...
ICO_ICON = ...
ICO_INFO = ...
ICO_APPMOD_LANGUAGE = ...
ICO_MAXIMIZE = ...
ICO_MINIMIZE = ...
ICO_MORE = ...
ICO_NORMALIZE = ...
ICO_QUITE = ...
ICO_SETTING = ...
ICO_USER = ...
ICO_HEBILL = ...
ICO_YANGS = ...


class Image(He):
    def __init__(self, u, source:str): ...

    @property
    def source(self)-> str: return ...
    @property
    def bytes_io(self)-> BytesIO: ...
    @property
    def bites(self)-> bytes: ...
    ##########
    def output_pil_image_obj(self, width: int = None, height: int = None)-> PilImage.Image: ...
    def save_by_pil_image(self, file: str | BytesIO, width: int = None, height: int = None)-> PilImage.Image: ...

    def pil_icons(self, file: str = None, sizes: list | tuple = None):...
    def save_as_pil_icons(self, file: str = None, sizes: list | tuple = None):...
    # 使用 wx.Image
    def wx_image(self, width: int = None, height: int = None):...
    def save_as_wx_image(self, file: str | BytesIO = None, width: int = None, height: int = None):...
    # 使用 wx.Bitmap
    def wx_bitmap(self, width: int = None, height: int = None):...
    def save_as_wx_bitmap(self, file: str = None, width: int = None, height: int = None):...
    def wx_bitmap_io(self, width: int = None, height: int = None):...
    # 使用 wx.Icon
    def wx_icon(self, size: int = None):...
    def save_as_wx_icon(self, file: str = None, size: int = None): ...
    def wx_icons(self, sizes: list = None): ...


class IconClose(Image):
    def __init__(self, u): ...
class IconDemo(Image):
    def __init__(self, u):...
class IconDocument(Image):
    def __init__(self, u): ...
class IconHelp(Image):
    def __init__(self, u): ...
class IconHome(Image):
    def __init__(self, u): ...
class IconIcon(Image):
    def __init__(self, u): ...
class IconInfo(Image):
    def __init__(self, u): ...
class IconLangauge(Image):
    def __init__(self, u): ...
class IconMaximize(Image):
    def __init__(self, u): ...
class IconMinimize(Image):
    def __init__(self, u): ...
class IconMore(Image):
    def __init__(self, u): ...
class IconNormalize(Image):
    def __init__(self, u): ...
class IconQuite(Image):
    def __init__(self, u): ...
class IconSetting(Image):
    def __init__(self, u): ...
class IconUser(Image):
    def __init__(self, u): ...
class IconHebill(Image):
    def __init__(self, u): ...
class IconYangs(Image):
    def __init__(self, u): ...



class Condition(dict):
    def __init__(self, column, value, judge=None, logic=None, startwith=None, endwith=None):
        dict.__init__(self)
        self.update({
            "column": column,
            "value": value,
            "judge": '=' if judge is None else judge,
            "logic": 'AND' if logic is None or logic.upper == 'AND' else 'OR',
            "startwith": bool(startwith),
            "endwith": bool(endwith)
        })

    @property
    def column(self): return self["column"]
    @column.setter
    def column(self, value): self["column"] = value
    @property
    def value(self): return self["value"]
    @value.setter
    def value(self, _value): self["value"] = _value


    @property
    def judge(self): return self["judge"]
    @judge.setter
    def judge(self, value): self["judge"] = value
    def set_judge_equal(self): self.judge = '='
    def set_judge_un_equal(self): self.judge = '!='
    def set_judge_less_than(self): self.judge = '<'
    def set_judge_more_than(self): self.judge = '>'
    def set_judge_un_less_than(self): self.judge = '>='
    def set_judge_un_more_than(self): self.judge = '<='
    def set_judge_like(self): self.judge = 'LIKE'
    def set_judge_regexp(self): self.judge = 'REGEXP'
    # TODO 需要更新解析语言
    def set_judge_in_null(self): self.judge = 'IS NULL'
    # TODO 需要更新解析语言
    def set_judge_is_not_null(self): self.judge = 'IS NOT NULL'
    # TODO 需要更新解析语言
    def set_judge_in(self): self.judge = 'IN'
    # TODO 需要更新解析语言
    def set_judge_not_in(self): self.judge = 'NOT IN'
    # TODO 需要更新解析语言
    def set_judge_between_and(self): self.judge = 'BETWEEN AND'
    # TODO 需要更新解析语言
    def set_judge_spatial(self): self.judge = 'SPATIAL'
    # TODO 需要更新解析语言
    def set_judge_sounds_like(self): self.judge = 'SOUNDS LIKE'

    @property
    def logic(self): return self["logic"]
    @logic.setter
    def logic(self, value: str): self["logic"] = 'AND' if value.upper == 'AND' else 'OR',
    def set_logic_and(self): self["logic"] = 'AND'
    def set_logic_or(self): self["logic"] = 'OR'

    @property
    def startwith(self): return self["startwith"]
    @startwith.setter
    def startwith(self, value): self["startwith"] = value
    def set_startwith(self): self.startwith = True

    @property
    def endwith(self): return self["endwith"]
    @endwith.setter
    def endwith(self, value): self["endwith"] = value
    def set_endwith(self): self.endwith = True

    def output(self, logic_bridge_requeried=False):
        if not self.column:
            return " "
        s = f'`{self.column}` {self.judge} "'
        if self.judge == "LIKE" and not self.startwith:
            s += "%"
        s += str(self.value)
        if self.judge == "LIKE" and not self.endwith:
            s += "%"
        s += '"'
        if logic_bridge_requeried:
            s += " " + self.logic + " " + s
        return s



from types import ModuleType
from .cls_type.core import ClsType
from .css_maker import CssMaker
from .database import Database
from .debugs import Debugs
from .exe_packer import ExePacker
from .file import File
from .folder import Folder
from .htmdoc import Htmdoc
from .image import Image, IconYangs, IconHebill
from .mod_path import ModPath
from .mod_type import ModType
from .multilinguals import Multilinguals
from .obj_type import ObjType
from .web_system import WebSystem
from ..types.he import He
from .configs import Configs


class Components(He):
    def cls_type(self, pycls: He | type)-> ClsType: ...
    def configs(self, data=None, end_class=None, start_class=None, file=None)-> Configs: ...
    def css_maker(self, file: str | File = None)-> CssMaker: ...

    def debugs(self)-> Debugs: ...
    def database(self, host: str = None, user: str = None, password: str = None, dbname: str = None, port: int = None, prefix: str = None, connection: str = None, configs:dict=None)-> Database: ...

    def exe_packer(self, mainpy: str, website_module:str=None, one_file:bool=False, one_dir:bool=False, show_console:bool=False, pack_overwrite:bool=False)-> ExePacker: ...

    def file(self, path: str)-> File: ...
    def folder(self, path: str)-> Folder: ...

    def htmdoc(self)-> Htmdoc: ...

    def image(self, source: str)-> Image: ...

    def mod_path(self, name: str | ModuleType | type | He)-> ModPath: ...
    def mod_type(self, pymod: ModuleType | type | He)-> ModType: ...
    def multilinguals(self, end_class: type = None, start_class: type = None, folder: Folder | str = None, hooked_configs: Configs = None, hooked_config_name: str = None)-> Multilinguals: ...

    def obj_type(self, pyobj: He)-> ObjType: ...

    def web_system(self, module: str, host: str=None, port: int=None, db_host: str=None, db_user: str=None, db_password: str=None, db_dbname: str=None, db_port: int=None, db_prefix: str=None, configs:dict=None)-> WebSystem: ...

    @property
    def image_icon_yangs(self) -> IconYangs: ...
    @property
    def image_icon_hebill(self) -> IconHebill: ...



from ...configs import Configs as PluCfg


class Configs(PluCfg):
    @property
    def prefix(self)-> str: ...
    @prefix.setter
    def prefix(self, value: str): ...
    @property
    def printable(self)-> bool: ...
    @printable.setter
    def printable(self, value: str): ...
    @property
    def saveable(self)-> bool: ...
    @saveable.setter
    def saveable(self, value: str): ...


from colorama import Fore, Style
from ....types.he import He


class Input(He):
    def input(self, label=None, prefix=None, color=None, bgcolor=None, bright=None):
        text = Fore.BLUE + self.__u__.make_prefix(prefix)
        if label:
            text += f'{label}'
        text += ': '
        if bgcolor:
            text += bgcolor
        if color:
            text += color
        if bright:
            text += Style.BRIGHT
        text += Style.RESET_ALL
        return input(text)

    def primary(self, label=None, prefix=None): return self.input(label, prefix, Fore.WHITE)
    def secondary(self, label=None, prefix=None): return self.input(label, prefix, Fore.BLUE)
    def success(self, label=None, prefix=None): return self.input(label, prefix, Fore.GREEN)
    def warning(self, label=None, prefix=None): return self.input(label, prefix, Fore.YELLOW)
    def danger(self, label=None, prefix=None): return self.input(label, prefix, Fore.RED)
    def info(self, label=None, prefix=None): return self.input(label, prefix, Fore.CYAN)
    def comment(self, label=None, prefix=None): return self.input(label, prefix, Fore.BLACK)



from ....types.hd import Hd
from ...multilinguals import Multilinguals
from ...file import File


class Package(Hd):
    def __init__(self, multilingual: Multilinguals, language: str): ...

    @property
    def __u__(self)-> Multilinguals: ...
    @property
    def language(self)-> str: ...
    @property
    def file(self)-> File: ...

    def read(self, code: str)-> str | None: ...
    def ml(self, code: str, replace: str = None, return_code: bool = False)-> str: ...

    def save(self, code: str | dict[str, str], traslation: str = None)-> None: ...



from ......functions.decorators import parse_self_x_item_value_or_set_by_func, parse_self_x_item_value
from ...connection import Connection
import pymysql
import pymysql.cursors
from dbutils.pooled_db import PooledDB


class Mysql(Connection):
    ########## 使用连接池 ##########
    @property
    @parse_self_x_item_value_or_set_by_func()
    def __pool__(self):
        return PooledDB(
            creator=pymysql,
            maxconnections=self.__u__.configs['max_connections'],
            host=self.__u__.configs['host'],
            user=self.__u__.configs['user'],
            password=self.__u__.configs['password'],
            port=self.__u__.configs['port'],
            database=self.__u__.configs['dbname'],
            charset='utf8mb4',
            cursorclass=pymysql.cursors.DictCursor,
            connect_timeout=self.__u__.configs['connect_timeout']
        )
    @property
    @parse_self_x_item_value()
    def __is_connected__(self): return

    # 平台函数
    def is_connected(self, recheck=False) :
        if not (self.__u__.configs['host'] and self.__u__.configs['port'] and self.__u__.configs['user'] and self.__u__.configs['password'] and self.__u__.configs['dbname']):
            self.__x__['__is_connected__'] = False
        elif recheck or self.__is_connected__ is None:
            try:
                conn = self.__pool__.connection()
                with conn.cursor() as cursor:
                    cursor.execute("SELECT 1")
                    cursor.fetchone()
                conn.close()
                self.__x__['__is_connected__'] = True
            except Exception:
                self.__x__['__is_connected__'] = False
        return self.__is_connected__

    def query(self, sql):
        self.queried_sqls.append(sql)
        info = f'数据库[{self.__u__.configs['host']}:{self.__u__.configs['port']}/{self.__u__.configs['dbname']}]查询'
        if self.is_connected():
            with self.__pool__.connection() as connection:
                with connection.cursor() as cursor:
                    cursor.execute(sql)
                    connection.commit()
                    connection.close()
                    self.__s__.debugs.output.success(f'{info}成功：{sql}')
                    return cursor
            """try:
                with self.__pool__.connection() as connection:
                    with connection.cursor() as cursor:
                        cursor.execute(sql)
                        connection.commit()
                        connection.close()
                        self.__s__.debugs.output.success(f'{info}成功：{sql}')
                        return cursor
            except Exception:
                return None"""
        return None

    def search_tables(self, keyword=None, startwith=None, endwith=None):
        sql =  f'SELECT TABLE_NAME FROM INFORMATION_SCHEMA.TABLES WHERE TABLE_SCHEMA = "{self.__u__.configs['dbname']}"'
        if keyword: sql += f' AND TABLE_NAME LIKE "{'' if startwith else '%'}{keyword}{'' if endwith else '%'}";'
        response = self.query(sql)
        result = []
        if response is not None:
            for row in response:
                result.append(row['TABLE_NAME'])
        return result

    def create_table(self, table):
        return self.query((
            f'CREATE TABLE IF NOT EXISTS `{table}` ('
            f'`{self.__u__.configs['column_key_sn']}` bigint NOT NULL AUTO_INCREMENT, '
            f'`{self.__u__.configs['column_key_id']}` varchar(64) UNIQUE NOT NULL, '
            f'`{self.__u__.configs['column_key_az']}` bigint NULL, '
            f'`{self.__u__.configs['column_key_kw']}` text NULL, '
            f'`{self.__u__.configs['column_key_ur']}` varchar(64) NOT NULL, '
            f'`{self.__u__.configs['column_key_dc']}` bigint NOT NULL, '
            f'`{self.__u__.configs['column_key_du']}` bigint NOT NULL, '
            f'PRIMARY KEY (`{self.__u__.configs['column_key_sn']}`)) ENGINE=InnoDB DEFAULT CHARACTER SET=utf8 COLLATE=utf8_unicode_ci'
            )) is not None

    def delete_table(self, table): return self.query(f'DROP TABLE IF EXISTS `{table}`') is not None

    def update_table_auto_increment(self, table, start=0):
        return self.query(f'ALTER TABLE {table} AUTO_INCREMENT = {start}') is not None

    def search_table_column_structures(self, table, keyword=None, startwith=None, endwith=None):
        sql = f'SHOW COLUMNS FROM {table}'
        if keyword: sql += f' LIKE "{'' if startwith else '%'}{keyword}{'' if endwith else '%'}";'
        return self.search_table_data_multiple(sql)

    def search_table_columns(self, table, keyword=None, startwith=None, endwith=None):
        return [i['Field'] for i in self.search_table_column_structures(table, keyword, startwith, endwith)]

    def create_table_column(self, table, column, datatype=None, length=0, unique=False, nullable=False):
        datatype = datatype.upper() if datatype and datatype.upper() in self.__s__.__i__.DATCOLS else 'varchar'
        sql = f"ALTER TABLE `{table}` ADD `{column}` {datatype}"
        if datatype == 'char' or datatype == 'varchar': sql += f'({length if length else 255})'
        sql += ' NULL' if nullable else ' NOT NULL;'
        result = self.query(sql)
        if result and unique: self.set_table_column_quniue(table, column)
        return result is not None

    def set_table_column_quniue(self, table, column): return self.query(f'ALTER TABLE `{table}` ADD UNIQUE (`{column}`);') is not None

    def search_table_data_multiple(self, sql):
        cursor = self.query(sql)
        if cursor is None: return []
        return list(cursor.fetchall())



from .navbar import Navbar
from ......types.hd import Hd
from .breadcrumb import Breadcrumb


class Caches(Hd):

    @property
    def breadcrumb(self)-> Breadcrumb: ...
    @property
    def navbar(self)-> Navbar: ...



from ..title import Title
from ... import Htmdoc
from ...nodes.tag import Tag
from ...nodes.wrap import Wrap


class Head(Tag):
    def __init__(self, u: Htmdoc | Wrap, attributes:dict[str, str|int|float|None]=None): ...
    @property
    def title(self)-> Title: ...





class Columns(list):
    def __init__(self, columns=None):
        list.__init__(self)
        self.add(columns)

    def add(self, column):
        if isinstance(column, str) and column and column not in self:
            self.append(column.lower)
        elif isinstance(column, list):
            for c in column:
                self.add(c)

    def output(self):
        if len(self) < 1:
            return "*"
        return ", ".join(self)



from ...types.he import He
from .configs import Configs
from .input import Input
from .output import Output

class Debugs(He):
    @property
    def configs(self)-> Configs: ...
    @property
    def input(self)-> Input: ...
    @property
    def output(self)-> Output: ...
    @property
    def ___prefixes___(self)-> list[str]: ...
    @property
    def prefixes(self)-> list[str]: ...
    def add_prefix(self, prefix: str): ...
    def make_prefix(self, prefix: str | list[str]): ...



from typing import Any
from .. import Client
from ....configs import Configs as WebSystemConfigs
from .....configs import Configs


class Address(Configs):
    def __init__(self, client: Client, mode: str = None, module: str = None, arguments: dict[str,str] = None, keyword: str = None, page: int = None, orders: dict[str,str] = None, filters: dict[str,str] = None, items_per_page: int = None, pages_per_group: int = None, reference: str = None, language: str = None, security: str = None): ...

    @property
    def __client__(self)-> Client: ...
    @property
    def ___websystem_configs___(self)-> WebSystemConfigs: ...

    @property
    def language(self) -> str:...
    @language.setter
    def language(self, language: str) -> None: ...
    def set_language(self, language: str=None): ...
    def is_language_chinese(self) -> bool: ...
    def is_language_english(self) -> bool: ...
    def set_language_chinese(self) -> None: ...
    def set_language_english(self) -> None: ...

    @property
    def mode(self) -> str:...
    @mode.setter
    def mode(self, mode: str) -> None: ...
    def is_mode_page(self) -> bool: ...
    def is_mode_json(self) -> bool: ...
    def is_mode_ajax(self) -> bool: ...
    def is_mode_grid(self) -> bool: ...
    def is_mode_file(self) -> bool: ...
    def is_mode_not_page(self) -> bool: ...
    def is_mode_not_json(self) -> bool: ...
    def is_mode_not_ajax(self) -> bool: ...
    def is_mode_not_grid(self) -> bool: ...
    def is_mode_not_file(self) -> bool: ...

    @property
    def module(self) -> str:...
    @module.setter
    def module(self, value: str) -> None: ...

    @property
    def security(self) -> str:...
    @security.setter
    def security(self, value: str) -> None: ...

    @property
    def arguments(self) -> dict:...
    @arguments.setter
    def arguments(self, value: dict) -> None: ...

    @property
    def keyword(self) -> str:...
    @keyword.setter
    def keyword(self, value: str) -> None: ...

    @property
    def page(self) -> int:...
    @page.setter
    def page(self, value: int) -> None: ...

    @property
    def orders(self) -> dict[str,str]:...
    @orders.setter
    def orders(self, value: dict[str,str]) -> None: ...

    @property
    def filters(self) -> dict[str,str]:...
    @filters.setter
    def filters(self, value: dict[str,str]) -> None: ...

    @property
    def items_per_page(self) -> int:...
    @items_per_page.setter
    def items_per_page(self, value: int) -> None: ...

    @property
    def pages_per_group(self) -> int:...
    @pages_per_group.setter
    def pages_per_group(self, value: int) -> None: ...

    @property
    def reference(self) -> str:...
    @reference.setter
    def reference(self, value: str) -> None: ...
    def update(self, m: dict, /, **kwargs): ...
    def copy(self)-> Address: ...
    def output(self)-> str: ...
    def data(self)-> dict[str, Any]: ...
    def redirect(self): ...



from ....types.hd import Hd
from .client import Client


class Clients(Hd):
    def __getitem__(self, key):
        if key not in self:
            self[key] = Client(self, key)
        return super().__getitem__(key)

    @property
    def __websystem__(self): return self.__u__

    def get(self, key): return self.__getitem__(key)



from ....types.he import He
from .. import WebSystem
from ..configs import Configs


class Server(He):
    def __init__(self, system: WebSystem): ...

    @property
    def __websystem__(self)-> WebSystem: ...
    @property
    def __configs__(self)-> Configs: ...
    def __deubg_warning__(self, content=None, error=None, comment=None, file=None)-> None: ...
    def __deubg_info__(self, content=None, error=None, comment=None, file=None)-> None: ...

    def make_response_text_html(self, code: str): ...
    def make_response_text_plain(self, text: str): ...
    def make_response_text_css(self, code: str): ...
    def make_response_text_csv(self, code: str): ...
    def make_response_text_markdown(self, code: str): ...
    def make_response_text_xml(self, code: str): ...

    def make_response_error(self, title: str = None, heading: str = None, content: str = None): ...
    def make_response_redirect(self, url): ...
    def make_response_file(self, file, as_attachment=False, mimetype=None, download_name=None): ...
    def make_response_404(self): ...
    @property
    def root_url(self)-> str: ...
    def generate_url(self, *names)-> str: ...
    def run(self): ...



from ..nodes.comment import Comment
from ..nodes.script import Script
from ..nodes.text import Text
from ..nodes.wrap import Wrap
from .. import Htmdoc
from ....types.he import He


class Node(He):
    def __init__(self, u: Htmdoc | Wrap) -> None: ...

    @property
    def id(self)-> int: ...

    @property
    def document(self)-> Htmdoc: ...
    def ___senior___(self, senior=None)-> bool: ...
    @property
    def senior(self)-> Wrap | None: ...
    def has_senior(self)-> bool: ...

    def output(self)-> str:...

    @staticmethod
    def is_comment()-> bool: ...
    @staticmethod
    def is_script()-> bool: ...
    @staticmethod
    def is_tag()-> bool: ...
    @staticmethod
    def is_text()-> bool: ...
    @staticmethod
    def is_wrap()-> bool: ...

    @property
    def ___anchored_to___(self)-> Node | None: ...
    def ___anchor_to___(self, node: Node): ...
    def ___is_anchored_to___(self)-> bool: ...
    @property
    def ___anchors___(self)-> dict[int, Wrap]: ...
    def ___anchor___(self, sn:int)-> Wrap: ...
    def before(self, sn:int)-> Wrap: ...
    @property
    def before1(self)-> Wrap: ...
    @property
    def before2(self)-> Wrap: ...
    @property
    def before3(self)-> Wrap: ...
    @property
    def before4(self)-> Wrap: ...
    @property
    def before5(self)-> Wrap: ...
    def after(self, sn:int)-> Wrap: ...
    @property
    def after1(self)-> Wrap: ...
    @property
    def after2(self)-> Wrap: ...
    @property
    def after3(self)-> Wrap: ...
    @property
    def after4(self)-> Wrap: ...
    @property
    def after5(self)-> Wrap: ...

    ####################################################################################################
    # Content 专用
    @property
    def ___content___(self)-> str: ...
    @___content___.setter
    def ___content___(self, content: str)-> Node: ...
    ####################################################################################################
    # Wrap 专用
    @property
    def ___juniors___(self)-> dict[int, Node]: ...

    def ___comment___(self, content: str = None)-> Comment: ...
    def ___script___(self, content: str = None)-> Script: ...
    def ___text___(self, content: str = None)-> Text: ...
    def ___wrap___(self, content=None, associated:Node=None)-> Wrap: ...

    def ___append___(self, element=None)-> Node | list[Node] | None: ...

    ####################################################################################################
    # Tag 专用
    @property
    def ___name___(self)-> str: ...
    @___name___.setter
    def ___name___(self, name: str)-> Node: ...
    @property
    def ___attributes___(self)-> dict[str, str|int|float|None]: ...

    @property
    def ___inner_anchored_to___(self)-> Node | None: ...
    def ___inner_anchor_to___(self, node: Node): ...
    def ___is_inner_anchored_to___(self)-> bool: ...
    @property
    def ___inner_anchors___(self)-> dict[int, Wrap]: ...
    def ___inner_anchor___(self, sn:int)-> Wrap: ...
    def ___inner_before___(self, sn:int)-> Wrap: ...
    @property
    def ___inner_before1___(self)-> Wrap: ...
    @property
    def ___inner_before2___(self)-> Wrap: ...
    @property
    def ___inner_before3___(self)-> Wrap: ...
    @property
    def ___inner_before4___(self)-> Wrap: ...
    @property
    def ___inner_before5___(self)-> Wrap: ...
    def ___inner_after___(self, sn:int)-> Wrap: ...
    @property
    def ___inner_after1___(self)-> Wrap: ...
    @property
    def ___inner_after2___(self)-> Wrap: ...
    @property
    def ___inner_after3___(self)-> Wrap: ...
    @property
    def ___inner_after4___(self)-> Wrap: ...
    @property
    def ___inner_after5___(self)-> Wrap: ...

    @property
    def ___output_pairable___(self)-> bool: ...
    @___output_pairable___.setter
    def ___output_pairable___(self, pairable: str)-> Node: ...
    @property
    def ___output_brealable___(self)-> bool: ...
    @___output_brealable___.setter
    def ___output_brealable___(self, pairable: str)-> Node: ...
    ####################################################################################################

    @property
    def ___level___(self)-> int: ...
    @property
    def ___output_level___(self)-> int: ...